#include <ECF/ECF.h>
#include "examples\COCO\FunctionMinEvalOp.h"




class HSAlg : public Algorithm
{
protected:
	// declare all available selection operators (not all get used)
	SelFitnessProportionalOpP selFitOp_;
	SelRandomOpP selRandomOp_;
	SelBestOpP selBestOp_;
	SelWorstOpP selWorstOp_;

	uint HMS;
	double HMCR, PAR, fw;
	double ubound, lbound;
	uint dimension;


public:
	// mandatory: define name, construct selection operators
	HSAlg()
	{
		name_ = "HSAlg";
		//selFitOp_ = (SelFitnessProportionalOpP) (new SelFitnessProportionalOp);
		selRandomOp_ = (SelRandomOpP)(new SelRandomOp);
		selBestOp_ = (SelBestOpP)(new SelBestOp);
		selWorstOp_ = (SelWorstOpP)(new SelWorstOp);
	}

	// optional: register any parameters
	void registerParameters(StateP state)
	{
		//define parameters
		registerParameter(state, "HMCR", (voidP) new double(0.9), ECF::DOUBLE);
		registerParameter(state, "PAR", (voidP) new double(0.3), ECF::DOUBLE);
		registerParameter(state, "fw", (voidP) new double(0.05), ECF::DOUBLE);
	}

	// optional: initialize components, read parameters
	bool initialize(StateP state)
	{
		// selection operators must be initialized if used!
		//selFitOp_->initialize(state);
		// optional: set ratio between the best and the worst individual's selection probability
		//selFitOp_->setSelPressure(10);
		// the following would cause the selection to be inverted fitness (badness) proportional
		//selFitOp_->setInverseProportional();
		//selRandomOp_->initialize(state);
		selBestOp_->initialize(state);
		selWorstOp_->initialize(state);

		// get parameters 
		voidP hmcr = getParameterValue(state, "HMCR");
		HMCR = *((double*)hmcr.get());
		
		voidP par = getParameterValue(state, "PAR");
		PAR = *((double*)par.get());
		
		voidP FW = getParameterValue(state, "fw");
		fw = *((double*)FW.get());
		
		// check if genotype is of a specific kind
		// algorithm accepts a single FloatingPoint Genotype
		FloatingPointP flp(new FloatingPoint::FloatingPoint);
		if (state->getGenotypes()[0]->getName() != flp->getName()) {
			ECF_LOG_ERROR(state, "Error: Harmony Search algorithm accepts only a single FloatingPoint genotype!");
			throw ("");
		}
		
		// read the dimension and domain boundaries
		voidP lBound = state->getGenotypes()[0]->getParameterValue(state, "lbound");
		lbound = *((double*)lBound.get());
		
		voidP uBound = state->getGenotypes()[0]->getParameterValue(state, "ubound");
		ubound = *((double*)uBound.get());
		
		voidP Dimension = state->getGenotypes()[0]->getParameterValue(state, "dimension");
		dimension = *((uint*)Dimension.get());
		

		// read population (local deme) size
		//uint HMS = state->getPopulation()->getLocalDeme()->getSize();
		
		return true;

	}

	// mandatory: perform single 'generation' (however the algorithm defines it)
	bool advanceGeneration(StateP state, DemeP deme)
	{
		uint HMS = deme->size();
		
		IndividualP NewVector = (IndividualP)deme->at(0)->copy();
		
		//FloatingPoint::FloatingPoint* flp = static_cast<FloatingPoint::FloatingPoint*> (NewVector->getGenotype().get());
		//FloatingPointP flp = boost::dynamic_pointer_cast<FloatingPoint::FloatingPoint> (NewVector->getGenotype(0));
		FloatingPointP flp = boost::static_pointer_cast<FloatingPoint::FloatingPoint> (NewVector->getGenotype(0));

		std::vector< double > &newvectval = flp->realValue;
		
		uint sizeofVect = (uint)newvectval.size();
		
		for (uint j = 0; j < sizeofVect; j++) {
			double randHMCR = state->getRandomizer()->getRandomDouble();  // rand[0,1]
			if (randHMCR <= HMCR) {
				
				// pick the stored value from HM, deme
				int randBR = state->getRandomizer()->getRandomInteger(HMS);  // rand[0, HMS-1]
				IndividualP temp = (IndividualP)deme->at(randBR);
				FloatingPointP traz = boost::static_pointer_cast<FloatingPoint::FloatingPoint> (temp->getGenotype(0));
				flp->realValue[j] = traz->realValue[j]; // HM[randBR][j]

				double randPAR = state->getRandomizer()->getRandomDouble();   // rand[0,1]
				if (randPAR <= PAR) {
					double randBR = (state->getRandomizer()->getRandomDouble()) * 2 - 1;    // rand[-1, 1]
					if (flp->realValue[j] + fw*randBR >= lbound && flp->realValue[j] + fw*randBR <= ubound) {
						flp->realValue[j] = flp->realValue[j] + fw*randBR;
					}
				} //else do nothing
			}
			else {
				//pick a random value within the allowed range
				double randVal = (state->getRandomizer()->getRandomDouble()) * (ubound - lbound) + lbound;  // rand[lbound,ubound]
				flp->realValue[j] = randVal;
			}

		}
		//novi vektor je modificiran ako je trebao bit, ovisno o vjerojatnostima
		// sad jo treba zamijeniti s najgorim iz memorije ako je taj novi bolji od trenutno najgoreg

		evaluate(NewVector);
		IndividualP najgori = selWorstOp_->select(*deme);
		evaluate(najgori);

		if (NewVector->fitness->isBetterThan(najgori->fitness))
		{
			replaceWith(najgori, NewVector);
		}

		return true;
	}
};
typedef boost::shared_ptr<HSAlg> HSAlgP;



// for a single function
int main(int argc, char **argv) {

	
	StateP state(new State);
	

	HSAlgP alg = (HSAlgP) new HSAlg;
	state->addAlgorithm(alg);
	
	state->setEvalOp(new FunctionMinEvalOp);
	
	state->initialize(argc, argv);
	
	state->run();
	
	return 0;
}
/*
//
// this main() function iterates over multiple COCO functions and optimizes each one in turn
// function Ids: noiseless 1-24, noisy 101-130
//

int main(int argc, char **argv)
{
// run for selected COCO functions
for(uint function = 21; function <= 24; function++) {

// read XML config
std::ifstream fin(argv[1]);
if (!fin) {
std::cerr << "Error opening config file! (" << argv[1] << ")\n";
return 1;
}

std::string xmlFile, temp;
while (!fin.eof()) {
getline(fin, temp);
xmlFile += "\n" + temp;
}
fin.close();

// set log and stats parameters
std::string funcName = uint2str(function);
std::string logName = "log", statsName = "stats";
if(function < 10) {
logName += "0";
statsName += "0";
}
logName += uint2str(function) + ".txt";
statsName += uint2str(function) + ".txt";

// update in XML
XMLResults results;
XMLNode xConfig = XMLNode::parseString(xmlFile.c_str(), "ECF", &results);
XMLNode registry = xConfig.getChildNode("Registry");

XMLNode func = registry.getChildNodeWithAttribute("Entry", "key", "coco.function");
func.updateText(funcName.c_str());
XMLNode log = registry.getChildNodeWithAttribute("Entry", "key", "log.filename");
log.updateText(logName.c_str());
XMLNode stats = registry.getChildNodeWithAttribute("Entry", "key", "batch.statsfile");
stats.updateText(statsName.c_str());

// write back
std::ofstream fout(argv[1]);
fout << xConfig.createXMLString(true);
fout.close();


// finally, run ECF on a single function
StateP state (new State);

//set my alg
HSAlgP alg = (HSAlgP) new HSAlg;
state->addAlgorithm(alg);

// set the evaluation operator
state->setEvalOp(new FunctionMinEvalOp);

state->initialize(argc, argv);
state->run();
}

return 0;
}
*/
